// Generated by CoffeeScript 1.10.0
(function() {
  var Comment, Raw,
    extend = function(child, parent) { for (var key in parent) { if (hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; },
    hasProp = {}.hasOwnProperty;

  Raw = require("./raw");

  module.exports = Comment = (function(superClass) {
    extend(Comment, superClass);

    function Comment() {
      return Comment.__super__.constructor.apply(this, arguments);
    }

    Comment.prototype.render = function() {
      return "";
    };

    return Comment;

  })(Raw);

}).call(this);

//# sourceMappingURL=comment.js.map
