// Generated by CoffeeScript 1.10.0
(function() {
  var Liquid, Unless,
    extend = function(child, parent) { for (var key in parent) { if (hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; },
    hasProp = {}.hasOwnProperty;

  Liquid = require("../../liquid");

  module.exports = Unless = (function(superClass) {
    extend(Unless, superClass);

    function Unless() {
      return Unless.__super__.constructor.apply(this, arguments);
    }

    Unless.prototype.parse = function() {
      return Unless.__super__.parse.apply(this, arguments).then((function(_this) {
        return function() {
          return _this.blocks[0].negate = true;
        };
      })(this));
    };

    return Unless;

  })(Liquid.If);

}).call(this);

//# sourceMappingURL=unless.js.map
