// Generated by CoffeeScript 1.10.0
(function() {
  var ElseCondition, Liquid,
    extend = function(child, parent) { for (var key in parent) { if (hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; },
    hasProp = {}.hasOwnProperty;

  Liquid = require("../liquid");

  module.exports = ElseCondition = (function(superClass) {
    extend(ElseCondition, superClass);

    function ElseCondition() {
      return ElseCondition.__super__.constructor.apply(this, arguments);
    }

    ElseCondition.prototype.evaluate = function() {
      return true;
    };

    return ElseCondition;

  })(Liquid.Condition);

}).call(this);

//# sourceMappingURL=else_condition.js.map
