// Generated by CoffeeScript 1.10.0
(function() {
  var Increment, Liquid,
    extend = function(child, parent) { for (var key in parent) { if (hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; },
    hasProp = {}.hasOwnProperty;

  Liquid = require("../../liquid");

  module.exports = Increment = (function(superClass) {
    extend(Increment, superClass);

    function Increment(template, tagName, markup) {
      this.variable = markup.trim();
      Increment.__super__.constructor.apply(this, arguments);
    }

    Increment.prototype.render = function(context) {
      var base, name, value;
      value = (base = context.environments[0])[name = this.variable] != null ? base[name] : base[name] = 0;
      context.environments[0][this.variable] = value + 1;
      return String(value);
    };

    return Increment;

  })(Liquid.Tag);

}).call(this);

//# sourceMappingURL=increment.js.map
